var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import * as React from 'react';
import PeriqlesField from './PeriqlesField';
import { introspect } from './functions';
import { commitMutation } from 'react-relay';
import '../periqles.css';
var useState = React.useState, useEffect = React.useEffect;
var PeriqlesForm = function (_a) {
    var environment = _a.environment, mutationName = _a.mutationName, mutationGQL = _a.mutationGQL, useMutation = _a.useMutation, specifications = _a.specifications, _b = _a.args, args = _b === void 0 ? {} : _b, callbacks = _a.callbacks;
    var _c = useState({}), formState = _c[0], setFormState = _c[1];
    var _d = useState([]), fields = _d[0], setFields = _d[1];
    // introspect this project's GraphQL schema on initial render
    useEffect(function () {
        introspect(mutationName, setFields, args);
    }, [mutationName]);
    // HANDLERS
    var initializeForm = function (fields) {
        var initialValues = {};
        fields.forEach(function (field) {
            var initialValue;
            switch (field.type) {
                case 'Enum':
                case 'String':
                    initialValue = '';
                    break;
                case 'Int':
                    initialValue = 0;
                    break;
                case 'Boolean':
                    initialValue = false;
                    break;
                default:
                    initialValue = '';
            }
            initialValues[field.name] = initialValue;
        });
        setFormState(initialValues);
    };
    var handleSubmit = function (e, fields) {
        if (e.key === 'Enter' || e.type === 'click') {
            e.preventDefault(); // prevent page refesh
        }
        // validate non-null fields
        var missing = [];
        var _loop_1 = function (key) {
            var fieldInfo = fields.filter(function (field) { return field.name === key; })[0];
            if (fieldInfo.required && formState[key] === ''
                || fieldInfo.required && formState[key] === undefined) {
                missing.push(fieldInfo.label);
            }
        };
        for (var key in formState) {
            _loop_1(key);
        }
        if (missing.length) {
            window.alert("The following fields are required: " + missing.join(', '));
            return;
        }
        var input = __assign(__assign({}, formState), args);
        var variables = {
            input: input,
        };
        if (environment) {
            // relay commit method
            commitMutation(environment, {
                mutation: mutationGQL,
                variables: variables,
                onCompleted: function (response, errors) {
                    if (callbacks === null || callbacks === void 0 ? void 0 : callbacks.onSuccess)
                        callbacks.onSuccess(response);
                    initializeForm(fields);
                },
                onError: function (err) {
                    if (callbacks === null || callbacks === void 0 ? void 0 : callbacks.onFailure)
                        callbacks.onFailure(err);
                },
            });
        }
        else {
            // apollo commit method
            useMutation({ variables: variables })
                .then(function (response) {
                if (callbacks === null || callbacks === void 0 ? void 0 : callbacks.onSuccess)
                    callbacks.onSuccess(response);
                initializeForm(fields);
            })
                .catch(function (err) {
                if (callbacks === null || callbacks === void 0 ? void 0 : callbacks.onFailure)
                    callbacks.onFailure(err);
            });
        }
    };
    var handleChange = function (e) {
        var _a = e.target, name = _a.name, value = _a.value, type = _a.type;
        var useValue = value;
        // type-coerce values from number input elements before storing in state
        if (type === 'number' && typeof value !== 'number') {
            useValue -= 0;
        }
        var newState = Object.assign({}, formState);
        newState[name] = useValue;
        setFormState(newState);
    };
    useEffect(function () { return initializeForm(fields); }, [fields.length, setFormState]);
    return (React.createElement("form", { className: "PeriqlesForm", "aria-labelledby": "form", onSubmit: function (e) { return handleSubmit(e, fields); } },
        specifications && specifications.header && React.createElement("h2", null, specifications.header),
        fields.length
            ? (fields.map(function (field, index) {
                var specs = specifications
                    ? specifications.fields[field.name]
                    : undefined;
                return (React.createElement(PeriqlesField, { key: "Periqles" + mutationName + "Field" + index, field: field, specs: specs, formState: formState, setFormState: setFormState, handleChange: handleChange }));
            }))
            : React.createElement("p", null, "Loading form..."),
        React.createElement("button", { className: "periqles-submit", onClick: function (e) { return handleSubmit(e, fields); } }, "Submit")));
};
export default PeriqlesForm;
//# sourceMappingURL=PeriqlesForm.js.map